# -*- coding: utf-8 -*-
"""
    zebe.service.blog
    ~~~~~~~~~~~~~~~~

    博客相关的服务

    :copyright: (c) 2018 by Zebe.
    :license: BSD, see LICENSE for more details.
"""

from zebe.modle.entity.blog import ZebeBlogArticle, BlogModelSession, CdrVbaBlogArticle, CdrVbaBlogSubscriber
from zebe.service.base import ModelBaseService


# Zebe博客文章服务
class ZebeBlogArticleService(ModelBaseService):
    def __init__(self):
        super().__init__(ZebeBlogArticle, BlogModelSession)


# CorelDrawVBA博客文章服务
class CdrVbaBlogArticleService(ModelBaseService):
    def __init__(self):
        super().__init__(CdrVbaBlogArticle, BlogModelSession)


# CorelDrawVBA博客订阅者服务
class CdrVbaBlogSubscriberService(ModelBaseService):
    def __init__(self):
        super().__init__(CdrVbaBlogSubscriber, BlogModelSession)

    # 通过邮箱查找一个订阅者
    def find_one_by_email(self, email):
        result = None
        if email is not None:
            result = self.session.query(self.entity).filter(self.entity.email == email).one_or_none()
            self.session.close()
        return result



# -*- coding: utf-8 -*-
"""
    zebe.config.db
    ~~~~~~~~~~~~~~~~

    数据库配置

    :copyright: (c) 2018 by Zebe.
    :license: BSD, see LICENSE for more details.
"""

import os

prefix = ''
if os.name == 'posix':  # Mac
    prefix = '/usr/zebe/'
elif os.name == 'linux':  # Linux
    prefix = '/home/data/'
elif os.name == 'nt':  # Windows
    prefix = 'D:/database/sqlite/'

WECHAT_DB = prefix + 'zebe-wechat.sqlite'  # 微信
PROJECT_DB = prefix + 'zebe-project.sqlite'  # 项目
MEMCARD_DB = prefix + 'zebe-memcard.sqlite'  # 秒记卡 TODO 合并到【记忆】
REMIND_DB = prefix + 'zebe-remind.sqlite'  # 提醒
BLOG_DB = prefix + 'zebe-blog.sqlite'  # 博客
DIARY_DB = prefix + 'zebe-diary.sqlite'  # 日记
AD_DB = prefix + 'zebe-ad.sqlite'  # 广告
APP_DB = prefix + 'zebe-app.sqlite'  # 应用
USER_DB = prefix + 'zebe-user.sqlite'  # 用户
MEMORY_DB = prefix + 'zebe-memory.sqlite'  # 记忆
ENGLISH_DB = prefix + 'zebe-english.sqlite'  # 英语
CDRVBA_DB = prefix + 'zebe-cdrvba.sqlite'  # cdrvba



# -*- coding: utf-8 -*-
"""
    zebe.model.entity.ad
    ~~~~~~~~~~~~~~~~~~~~

    广告相关的实体类

    :copyright: (c) 2018 by Zebe.
    :license: BSD, see LICENSE for more details.
"""
from datetime import datetime

from sqlalchemy import Column, String, Integer, DateTime, SmallInteger
from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker

from zebe.config.db import AD_DB

engine = create_engine('sqlite:///' + AD_DB, echo=True)
ModelBase = declarative_base()


# 广告
class Ad(ModelBase):
    __tablename__ = 'ad'
    id = Column(Integer, primary_key=True, autoincrement=True, index=True)  # ID
    name = Column(String(40), default="", nullable=False)  # 广告名称
    image = Column(String(300), default="", nullable=False)  # 广告图片
    link = Column(String(300), default="", nullable=False)  # 跳转链接
    is_enable = Column(SmallInteger, default=0, nullable=False, index=True)  # 是否有效（0=否 1=是）
    click_count = Column(Integer)  # 点击次数
    create_time = Column(DateTime, default=datetime.now(), nullable=False)  # 创建时间

    def __repr__(self):
        return "<%s(name='%s')>" % (self.__class__.name, self.name)


ModelBase.metadata.create_all(engine)
AdModelSession = sessionmaker(bind=engine)



# -*- coding: utf-8 -*-
"""
    zebe.model.entity.english
    ~~~~~~~~~~~~~~~~~~~~~

    英语相关的实体类

    :copyright: (c) 2018 by Zebe.
    :license: BSD, see LICENSE for more details.
"""

from sqlalchemy import Column, String, Integer
from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker

from zebe.config.db import ENGLISH_DB

engine = create_engine('sqlite:///' + ENGLISH_DB, echo=True)
ModelBase = declarative_base()


# 英语短语
class EnglishPhrase(ModelBase):
    __tablename__ = 'english_phrase'
    id = Column(Integer, primary_key=True, autoincrement=True, index=True)  # ID
    first_letter = Column(String(1), default="", nullable=False, index=True)  # 首字母
    word = Column(String(30), default="", nullable=False)  # 单词
    description = Column(String(50), default="", nullable=False)  # 含义
    demo = Column(String(100), default="", nullable=False)  # 例句

    def __repr__(self):
        return "<%s(name='%s')>" % (self.__class__.name, self.name)


ModelBase.metadata.create_all(engine)
EnglishModelSession = sessionmaker(bind=engine)



# -*- coding: utf-8 -*-
"""
    zebe.model.entity.memory
    ~~~~~~~~~~~~~~~~~~~~~~~

    记忆相关的实体类

    :copyright: (c) 2018 by Zebe.
    :license: BSD, see LICENSE for more details.
"""

from datetime import datetime

from sqlalchemy import Column, String, DateTime, Integer, Text, SmallInteger
from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker

from zebe.config.db import MEMORY_DB

engine = create_engine('sqlite:///' + MEMORY_DB, echo=True)
MemoryModelBase = declarative_base()


# 数字编码
class NumberCode(MemoryModelBase):
    __tablename__ = 'number_code'
    id = Column(Integer, primary_key=True, autoincrement=True, index=True)  # ID
    number = Column(String(3), default="", nullable=False, index=True)  # 数字
    code = Column(String(10), default="", nullable=False)  # 编码
    image = Column(String(300), default="", nullable=False)  # 图像
    version = Column(Integer, default=0, nullable=False)  # 版本
    create_time = Column(DateTime, default=datetime.now(), nullable=False)  # 创建时间

    def __repr__(self):
        return "<%s(number='%s')>" % (self.__class__.name, self.number)


MemoryModelBase.metadata.create_all(engine)
MemoryModelModelSession = sessionmaker(bind=engine)



# -*- coding: utf-8 -*-
"""
    zebe.model.entity.diary
    ~~~~~~~~~~~~~~~~~~~~~~~

    日记相关的实体类

    :copyright: (c) 2018 by Zebe.
    :license: BSD, see LICENSE for more details.
"""

from datetime import datetime

from sqlalchemy import Column, String, DateTime, Integer, Text, SmallInteger
from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker

from zebe.config.db import DIARY_DB

engine = create_engine('sqlite:///' + DIARY_DB, echo=True)
DiaryModelBase = declarative_base()


# 日记
class Diary(DiaryModelBase):
    __tablename__ = 'diary'
    id = Column(Integer, primary_key=True, autoincrement=True, index=True)  # ID
    title = Column(String(50), default="", nullable=False)  # 标题
    content = Column(Text, default="", nullable=False)  # 内容
    learned = Column(SmallInteger, default=0, nullable=False)  # 新掌握知识数量
    year = Column(SmallInteger, default=0, nullable=False, index=True)  # 所属年份
    month = Column(SmallInteger, default=0, nullable=False, index=True)  # 所属月份
    day = Column(SmallInteger, default=0, nullable=False, index=True)  # 所属日期
    create_time = Column(DateTime, default=datetime.now(), nullable=False)  # 创建时间

    def __repr__(self):
        return "<%s(title='%s')>" % (self.__class__.name, self.title)


DiaryModelBase.metadata.create_all(engine)
DiaryModelSession = sessionmaker(bind=engine)



# -*- coding: utf-8 -*-
"""
    zebe.service.diary
    ~~~~~~~~~~~~~~~~

    日记相关的服务

    :copyright: (c) 2018 by Zebe.
    :license: BSD, see LICENSE for more details.
"""
from datetime import datetime

from zebe.modle.entity.diary import Diary, DiaryModelSession
from zebe.service.base import ModelBaseService


# 日记服务
class DiaryService(ModelBaseService):
    def __init__(self):
        super().__init__(Diary, DiaryModelSession)

    def find_one_by_title(self, title):
        """
        通过标题查找一篇日记
        :param title: 标题
        :return:
        """
        result = None
        if title is not None:
            result = self.session.query(self.entity).filter(self.entity.title == title).one_or_none()
            self.session.close()
        return result

    def find_all_by_date(self, year, month, day):
        """
        按日期查询全部数据
        :param year: 年
        :param month: 月
        :param day: 日
        :return:
        """
        result = self.session.query(self.entity).filter(self.entity.year == year, self.entity.month == month,
                                                        self.entity.day == day).all()
        self.session.close()
        return result

    def find_all_by_year(self, year):
        """
        按年查询全部数据
        :param year: 年
        :return:
        """
        result = self.session.query(self.entity).filter(self.entity.year == year).all()
        self.session.close()
        return result

    def find_all_by_month(self, year, month):
        """
        按月查询全部数据
        :param year: 年
        :param month: 月
        :return:
        """
        result = self.session.query(self.entity).filter(self.entity.year == year, self.entity.month == month).all()
        self.session.close()
        return result

    def find_all_by_today(self):
        """
        查询今天全部数据
        :return:
        """
        now = datetime.now()
        year = now.year
        month = now.month
        day = now.day
        return self.find_all_by_date(year, month, day)

    def find_all_by_this_month(self):
        """
        查询本月的全部数据
        :return:
        """
        now = datetime.now()
        year = now.year
        month = now.month
        return self.find_all_by_month(year, month)

    def find_all_by_this_year(self):
        """
        查询本年的全部数据
        :return:
        """
        now = datetime.now()
        year = now.year
        return self.find_all_by_year(year)

    def find_all_order_by_date(self):
        """
        查询全部数据并指定排序
        :return:
        """
        result = self.session.query(self.entity).order_by(self.entity.create_time.desc()).all()
        self.session.close()
        return result

    @staticmethod
    def get_total_learn_count(diary_list):
        """
        计算一组日记列表的总收获数量
        :param diary_list: 日记列表
        :return:
        """
        total = 0
        if diary_list is not None:
            for diary in diary_list:
                total += diary.learned
        return total



# -*- coding: utf-8 -*-
"""
    zebe.model.entity.cdrvba
    ~~~~~~~~~~~~~~~~~~~~~~~

    CorelDrawVBA相关的实体类

    :copyright: (c) 2018 by Zebe.
    :license: BSD, see LICENSE for more details.
"""

from datetime import datetime

from sqlalchemy import Column, String, DateTime, Integer, Text
from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker

from zebe.config.db import CDRVBA_DB

engine = create_engine('sqlite:///' + CDRVBA_DB, echo=True)
CorelDrawVBAModelBase = declarative_base()


# CorelDrawVBA文章
class CorelDrawVBAArticle(CorelDrawVBAModelBase):
    __tablename__ = 'cdrvba_article'
    id = Column(Integer, primary_key=True, autoincrement=True, index=True)  # ID
    title = Column(String(50), default="", nullable=False)  # 标题
    summary = Column(String(150), default="", nullable=False)  # 摘要
    content = Column(Text, default="", nullable=False)  # 内容
    type = Column(String(10), default="", nullable=False, index=True)  # 分类
    tags = Column(String(100), default="", nullable=False)  # 标签
    cover = Column(String(300), default="", nullable=False)  # 封面图
    view_count = Column(Integer, default=0, nullable=False)  # 浏览数
    comment_count = Column(Integer, default=0, nullable=False)  # 评论数
    link = Column(String(300), default="", nullable=False, index=True)  # 链接
    create_time = Column(DateTime, default=datetime.now(), nullable=False)  # 创建时间


CorelDrawVBAModelBase.metadata.create_all(engine)
CorelDrawVBAModelSession = sessionmaker(bind=engine)



# -*- coding: utf-8 -*-
"""
    zebe.service.project
    ~~~~~~~~~~~~~~~~

    项目相关的服务

    :copyright: (c) 2018 by Zebe.
    :license: BSD, see LICENSE for more details.
"""

from datetime import datetime

from zebe.utils.datetime_util import get_total_day_of_year, get_which_day_of_year, get_today_str, get_tomorrow_str

from zebe.modle.entity.project import Project, ProjectModelSession, AutoDailyTask, Task
from zebe.service.base import ModelBaseService


# 项目服务
class ProjectService(ModelBaseService):
    def __init__(self):
        super().__init__(Project, ProjectModelSession)

    # 按照年份和类型查询全部项目
    def find_by_year_and_type(self, year, project_type):
        entity = self.entity
        result = self.session.query(entity).filter(entity.year == int(year), entity.type == project_type).all()
        self.session.close()
        return result

    # 按照年份查询全部项目
    def find_all_by_year(self, year):
        entity = self.entity
        result = self.session.query(entity).filter(entity.year == int(year)).all()
        self.session.close()
        return result

    # 按照年份查询主要项目
    def find_main_by_year(self, year):
        entity = self.entity
        result = self.session.query(entity).filter(entity.year == int(year), entity.is_main == 1).all()
        self.session.close()
        return result

    # 按照年份查询次要项目
    def find_secondary_by_year(self, year):
        entity = self.entity
        result = self.session.query(entity).filter(entity.year == int(year), entity.is_main == 0).all()
        self.session.close()
        return result

    # 按照年份和类型查询项目总数
    def count_by_year_and_type(self, year, project_type):
        entity = self.entity
        result = self.session.query(entity).filter(entity.year == int(year), entity.type == project_type).count()
        self.session.close()
        return result

    # 更新单个任务进度
    def update_progress(self, data_id, progress):
        entity = self.session.query(self.entity).filter(self.entity.id == data_id).one_or_none()
        if entity is not None:
            entity.progress = progress
            self.session.commit()
            self.session.close()

    # 按照名称查找项目
    def find_one_by_name(self, name):
        result = self.session.query(self.entity).filter(self.entity.name == name).one_or_none()
        self.session.close()
        return result

    # 获取一组项目的进度
    def get_progress_of_projects(self, project_list):
        progress = 0
        if project_list is not None and len(project_list) > 0:
            total_project = len(project_list)
            total_progress = 0
            for project in project_list:
                total_progress += project.progress
            progress = round(total_progress / total_project, 2)
        return progress

    # 获取一组项目的任务总数
    def get_total_task_of_projects(self, project_list):
        total_task = 0
        if project_list is not None and len(project_list) > 0:
            for project in project_list:
                total_task += project.total_task
        return total_task

    # 获取一组项目的已完成任务总数
    def get_finished_task_of_projects(self, project_list):
        total_finished_task = 0
        if project_list is not None and len(project_list) > 0:
            for project in project_list:
                total_finished_task += project.finished_task
        return total_finished_task


# 任务服务
class TaskService(ModelBaseService):
    def __init__(self):
        super().__init__(Task, ProjectModelSession)

    # 按照年份查询全部任务总数
    def count_all_by_year(self, year):
        entity = self.entity
        result = self.session.query(entity).filter(entity.year == int(year)).count()
        self.session.close()
        return result

    # 按照项目查询全部任务
    def find_all_by_project(self, project_id):
        entity = self.entity
        result = self.session.query(entity).filter(entity.project_id == int(project_id)).all()
        self.session.close()
        return result

    # 按照年份查询已完成任务总数
    def count_finished_by_year(self, year):
        # TODO 1 是个魔法数字，修正为常量
        entity = self.entity
        result = self.session.query(entity).filter(entity.year == int(year), entity.status == 1).count()
        self.session.close()
        return result

    # 获取截止到某一天应该完成的任务总数（按年计算任务量）
    def get_task_should_finished_by_day(self, year, month, day):
        total_days_of_year = get_total_day_of_year(year)
        which_day_in_year = get_which_day_of_year(year, month, day)
        total_task_of_year = self.count_all_by_year(year)
        today_should_finished_task = int((total_task_of_year / total_days_of_year) * which_day_in_year)
        return today_should_finished_task


# 自动日常任务服务
class AutoDailyTaskService(ModelBaseService):
    def __init__(self):
        super().__init__(AutoDailyTask, ProjectModelSession)

    def get_today_task_all(self):
        """
        查询今日全部任务
        :return:
        """
        today = datetime.now().strftime("%Y-%m-%d")
        return self.find_all_by_date(today)

    def get_today_task_unfinished(self):
        """
        查询今日未完成任务
        :return:
        """
        today = datetime.now().strftime("%Y-%m-%d")
        return self.find_unfinished_by_date(today)

    def find_one_by_title_and_date(self, title, date):
        """
        按标题和日期查询单个任务
        :param date:
        :return:
        """
        entity = self.entity
        result = self.session.query(entity).filter(entity.title == title, entity.date == str(date)).one_or_none()
        self.session.close()
        return result

    def find_all_by_date(self, date):
        """
        按日期查询全部任务
        :param date:
        :return:
        """
        entity = self.entity
        result = self.session.query(entity).filter(entity.date == str(date)).order_by(self.entity.status.asc(),
                                                                                      self.entity.is_main.desc(),
                                                                                      self.entity.idx_order.asc()).all()
        self.session.close()
        return result

    def find_all_unfinished(self):
        """
        查询所有未完成任务
        :return:
        """
        entity = self.entity
        result = self.session.query(entity).filter(entity.status == 0).all()
        self.session.close()
        return result

    # 按日期查询未完成任务
    def find_unfinished_by_date(self, date):
        entity = self.entity
        result = self.session.query(entity).filter(entity.date == str(date), entity.status == 0).order_by(self.entity.status.asc(),
                                                                                      self.entity.is_main.desc(),
                                                                                      self.entity.idx_order.asc()).all()
        self.session.close()
        return result

    # 按任务ID查询单个任务
    def find_one_by_task(self, task_id):
        result = self.session.query(self.entity).filter(self.entity.task_id == task_id).one_or_none()
        self.session.close()
        return result

    # 按日期删除全部数据
    def delete_all_by_date(self, date):
        self.session.query(self.entity).filter(self.entity.date == str(date)).delete(synchronize_session=False)
        self.session.commit()
        self.session.close()

    # 标记任务完成
    def mark_finish(self, data_id):
        self.__mark_finish_or_unfinish(data_id, True)

    # 标记任务未完成
    def mark_unfinish(self, data_id):
        self.__mark_finish_or_unfinish(data_id, False)

    # 标记任务完成|未完成
    def __mark_finish_or_unfinish(self, data_id, finished):
        entity = self.find_one(data_id)
        if entity is not None:
            task_id = entity.task_id
            project_id = entity.project_id
            entity.status = 1 if finished else 0  # TODO 1和0是魔法数字，改为常量
            entity.finish_time = datetime.now() if finished else None
            self.update(entity)
            # 如果任务是某个项目下的任务，则自动把任务标记为已完成
            if task_id is not None:
                task_service = TaskService()
                task = task_service.find_one(task_id)
                if task is not None:
                    task.status = 1 if finished else 0
                    task_service.update(task)
            # 如果任务归属于某个项目，则自动更新项目的进度
            if project_id is not None:
                project_service = ProjectService()
                project = project_service.find_one(project_id)
                if project is not None:
                    project.finished_task = project.finished_task + 1
                    project.progress = round((project.finished_task / project.total_task) * 100, 2)
                    project_service.update(project)
                    print("任务归属于项目，已自动更新项目的进度")
            else:
                print("任务不归属于任何项目，不需要更新项目的进度")

    def move_all_unfinished_task_to_today(self):
        """
        移动所有未完成任务到今天，对应任务的拖延次数加1
        :return:
        """
        task_list = self.find_all_unfinished()
        if len(task_list) > 0:
            today = get_today_str()
            for task in task_list:
                exist_task = self.find_one_by_title_and_date(task.title, today)
                if exist_task is None:
                    task.date = today
                    task.delay_count = task.delay_count + 1
                    print('过期任务[' + task.title + ']已经移动到今日任务列表')
                    self.update(task)

    def move_today_unfinished_task_to_tomorrow(self):
        """
        移动今日未完成任务到明天，对应任务的拖延次数加1
        :return:
        """
        task_list = self.get_today_task_unfinished()
        if len(task_list) > 0:
            tomorrow = get_tomorrow_str()
            for task in task_list:
                exist_task = self.find_one_by_title_and_date(task.title, tomorrow)
                if exist_task is None:
                    task.date = tomorrow
                    task.delay_count = task.delay_count + 1
                    print('过期任务[' + task.title + ']已经移动到明日任务列表')
                    self.update(task)
                else:
                    print('过期任务[' + task.title + ']已经存在于明日任务列表')
